@@ -16,15 +16,18 @@
 
 from __future__ import unicode_literals
 
-import copy
+from enum import IntEnum
 import json
+import logging
 import os
 import sys
 
 from turbinia import config
 from turbinia import TurbiniaException
+from turbinia.processors import docker
 from turbinia.processors import mount_local
 from turbinia.processors import archive
+from turbinia.lib.docker_manager import GetDockerPath
 
 # pylint: disable=keyword-arg-before-vararg
 
@@ -32,6 +35,8 @@ config.LoadConfig()
 if config.TASK_MANAGER.lower() == 'psq':
   from turbinia.processors import google_cloud
 
+log = logging.getLogger('turbinia')
+
 
 def evidence_decode(evidence_dict):
   """Decode JSON into appropriate Evidence object.
@@ -52,29 +57,53 @@ def evidence_decode(evidence_dict):
         'Evidence_dict is not a dictionary, type is {0:s}'.format(
             str(type(evidence_dict))))
 
-  type_ = evidence_dict.get('type', None)
+  type_ = evidence_dict.pop('type', None)
   if not type_:
     raise TurbiniaException(
         'No Type attribute for evidence object [{0:s}]'.format(
             str(evidence_dict)))
 
   try:
-    evidence = getattr(sys.modules[__name__], type_)()
+    evidence_class = getattr(sys.modules[__name__], type_)
+    evidence = evidence_class.from_dict(evidence_dict)
   except AttributeError:
     raise TurbiniaException(
         'No Evidence object of type {0:s} in evidence module'.format(type_))
 
-  evidence.__dict__ = evidence_dict
   if evidence_dict.get('parent_evidence'):
     evidence.parent_evidence = evidence_decode(evidence_dict['parent_evidence'])
   if evidence_dict.get('collection'):
     evidence.collection = [
         evidence_decode(e) for e in evidence_dict['collection']
     ]
+
+  # We can just reinitialize instead of deserializing because the state should
+  # be empty when just starting to process on a new machine.
+  evidence.state = {}
+  for state in EvidenceState:
+    evidence.state[state] = False
+
   return evidence
 
 
-class Evidence(object):
+class EvidenceState(IntEnum):
+  """Runtime state of Evidence.
+
+  Evidence objects will map each of these to a boolean indicating the current
+  state for the given object.
+  """
+  MOUNTED = 1
+  ATTACHED = 2
+  # Using PARENT_* states here for compound evidence types like
+  # GoogleCloudDiskRawEmbedded so that we can control those states
+  # independently.
+  PARENT_MOUNTED = 3
+  PARENT_ATTACHED = 4
+  DECOMPRESSED = 5
+  DOCKER_MOUNTED = 6
+
+
+class Evidence:
   """Evidence object for processing.
 
   In most cases, these objects will just contain metadata about the actual
@@ -98,7 +127,16 @@ class Evidence(object):
         to the saved_path location.
     source (str): String indicating where evidence came from (including tool
         version that created it, if appropriate).
-    local_path (str): A string of the local_path to the evidence.
+    local_path (str): Path to the processed data (can be a blockdevice or a
+        mounted directory, etc).  This is the path that most Tasks should use to
+        access evidence after it's been processed on the worker (i.e. when the
+        Tasks `run()` method is called).  The last pre-processor to run should
+        set this path.
+    source_path (str): Path to the original un-processed source data for the
+        Evidence.  This is the path that Evidence should be created and set up
+        with initially.  Tasks should generally not use this path, but instead
+        use the `local_path`.
+    mount_path (str): Path to a mounted file system (if relevant).
     tags (dict): Extra tags associated with this evidence.
     request_id (str): The id of the request this evidence came from, if any.
     parent_evidence (Evidence): The Evidence object that was used to generate
@@ -108,27 +146,39 @@ class Evidence(object):
         file alongside the Evidence when saving to external storage.  The
         metadata file will contain all of the key=value pairs sent along with
         the processing request in the recipe.  The output is in JSON format
+    state (dict): A map of each EvidenceState type to a boolean to indicate
+        if that state is true.  This is used by the preprocessors to set the
+        current state and Tasks can use this to determine if the Evidence is in
+        the correct state for processing.
   """
 
   # The list of attributes a given piece of Evidence requires to be set
   REQUIRED_ATTRIBUTES = []
 
+  # The list of EvidenceState states that the Evidence supports in its
+  # pre/post-processing (e.g. MOUNTED, ATTACHED, etc).  See `preprocessor()`
+  # docstrings for more info.
+  POSSIBLE_STATES = []
+
   def __init__(
-      self, name=None, description=None, source=None, local_path=None,
-      tags=None, request_id=None):
+      self, name=None, description=None, source=None, source_path=None,
+      tags=None, request_id=None, copyable=False):
     """Initialization for Evidence."""
-    self.copyable = False
+    self.copyable = copyable
     self.config = {}
     self.context_dependent = False
     self.cloud_only = False
     self.description = description
+    self.mount_path = None
     self.source = source
-    self.local_path = local_path
+    self.source_path = source_path
     self.tags = tags if tags else {}
     self.request_id = request_id
     self.parent_evidence = None
     self.save_metadata = False
 
+    self.local_path = source_path
+
     # List of jobs that have processed this evidence
     self.processed_by = []
     self.type = self.__class__.__name__
@@ -136,15 +186,45 @@ class Evidence(object):
     self.saved_path = None
     self.saved_path_type = None
 
+    self.state = {}
+    for state in EvidenceState:
+      self.state[state] = False
+
+    if self.copyable and not self.local_path:
+      raise TurbiniaException(
+          '{0:s} is a copyable evidence and needs a source_path'.format(
+              self.type))
+
   def __str__(self):
-    return '{0:s}:{1:s}:{2!s}'.format(self.type, self.name, self.local_path)
+    return '{0:s}:{1:s}:{2!s}'.format(self.type, self.name, self.source_path)
 
   def __repr__(self):
     return self.__str__()
 
+  @classmethod
+  def from_dict(cls, dictionary):
+    """Instanciate an Evidence object from a dictionary of attributes.
+
+    Args:
+      dictionary(dict): the attributes to set for this object.
+    Returns:
+      Evidence: the instantiated evidence.
+    """
+    name = dictionary.pop('name', None)
+    description = dictionary.pop('description', None)
+    source = dictionary.pop('source', None)
+    source_path = dictionary.pop('source_path', None)
+    tags = dictionary.pop('tags', None)
+    request_id = dictionary.pop('request_id', None)
+    new_object = cls(
+        name=name, description=description, source=source,
+        source_path=source_path, tags=tags, request_id=request_id)
+    new_object.__dict__.update(dictionary)
+    return new_object
+
   def serialize(self):
     """Return JSON serializable object."""
-    serialized_evidence = copy.deepcopy(self.__dict__)
+    serialized_evidence = self.__dict__.copy()
     if self.parent_evidence:
       serialized_evidence['parent_evidence'] = self.parent_evidence.serialize()
     return serialized_evidence
@@ -167,12 +247,16 @@ class Evidence(object):
 
     return serialized
 
-  def _preprocess(self, _):
+  def _preprocess(self, _, required_states):
     """Preprocess this evidence prior to task running.
 
-    This gets run in the context of the local task execution on the worker
-    nodes prior to the task itself running.  This can be used to prepare the
-    evidence to be processed (e.g. attach a cloud disk, mount a local disk etc).
+    See `preprocess()` docstrings for more info.
+
+    Args:
+      tmp_dir(str): The path to the temporary directory that the
+                       Task will write to.
+      required_states(list[EvidenceState]): The list of evidence state
+          requirements from the Task.
     """
     pass
 
@@ -185,21 +269,74 @@ class Evidence(object):
     """
     pass
 
-  def preprocess(self, tmp_dir=None):
+  def preprocess(self, tmp_dir=None, required_states=None):
     """Runs the possible parent's evidence preprocessing code, then ours.
 
     This is a wrapper function that will call the chain of pre-processors
     starting with the most distant ancestor.  After all of the ancestors have
-    been processed, then we run our pre-processor.
+    been processed, then we run our pre-processor.  These processors get run in
+    the context of the local task execution on the worker nodes prior to the
+    task itself running.  This can be used to prepare the evidence to be
+    processed (e.g. attach a cloud disk, mount a local disk etc).
+
+    Tasks export a list of the required_states they have for the state of the
+    Evidence it can process in `TurbiniaTask.REQUIRED_STATES`[1].  Evidence also
+    exports a list of the possible states it can have after pre/post-processing
+    in `Evidence.POSSIBLE_STATES`.  The pre-processors should run selectively
+    based on the these requirements that come from the Task, and the
+    post-processors should run selectively based on the current state of the
+    Evidence.
+
+    If a Task requires a given state supported by the given Evidence class, but
+    it is not met after the preprocessing of the Evidence is run, then the Task
+    will abort early.  Note that for compound evidence types that have parent
+    Evidence objects (e.g. where `context_dependent` is True), we only inspect
+    the child Evidence type for its state as it is assumed that it would only be
+    able to run the appropriate pre/post-processors when the parent Evidence
+    processors have been successful.
+
+    [1] Note that the evidence states required by the Task are only required if
+    the Evidence also supports that state in `POSSSIBLE_STATES`.  This is so
+    that the Tasks are flexible enough to support multiple types of Evidence.
+    For example, `PlasoTask` allows both `CompressedDirectory` and
+    `GoogleCloudDisk` as Evidence input, and has states `ATTACHED` and
+    `DECOMPRESSED` listed in `PlasoTask.REQUIRED_STATES`.  Since `ATTACHED`
+    state is supported by `GoogleCloudDisk`, and `DECOMPRESSED` is supported by
+    `CompressedDirectory`, only those respective pre-processors will be run and
+    the state is confirmed after the preprocessing is complete.
 
     Args:
       tmp_dir(str): The path to the temporary directory that the
                        Task will write to.
+      required_states(list[EvidenceState]): The list of evidence state
+          requirements from the Task.
 
+    Raises:
+      TurbiniaException: If the required evidence state cannot be met by the
+          possible states of the Evidence or if the parent evidence object does
+          not exist when it is required by the Evidence type..
     """
-    if self.parent_evidence:
-      self.parent_evidence.preprocess()
-    self._preprocess(tmp_dir)
+    self.local_path = self.source_path
+    if not required_states:
+      required_states = []
+
+    if self.context_dependent:
+      if not self.parent_evidence:
+        raise TurbiniaException(
+            'Evidence of type {0:s} needs parent_evidence to be set'.format(
+                self.type))
+      self.parent_evidence.preprocess(tmp_dir, required_states)
+    try:
+      log.debug('Starting pre-processor for evidence {0:s}'.format(self.name))
+      self._preprocess(tmp_dir, required_states)
+    except TurbiniaException as exception:
+      log.error(
+          'Error running preprocessor for {0:s}: {1!s}'.format(
+              self.name, exception))
+
+    log.debug(
+        'Pre-processing evidence {0:s} is complete, and evidence is in state '
+        '{1:s}'.format(self.name, self.format_state()))
 
   def postprocess(self):
     """Runs our postprocessing code, then our possible parent's evidence.
@@ -208,10 +345,23 @@ class Evidence(object):
     then recurse down the chain of parent Evidence and run those post-processors
     in order.
     """
+    log.info('Starting post-processor for evidence {0:s}'.format(self.name))
+    log.debug('Evidence state: {0:s}'.format(self.format_state()))
     self._postprocess()
     if self.parent_evidence:
       self.parent_evidence.postprocess()
 
+  def format_state(self):
+    """Returns a string representing the current state of evidence.
+
+    Returns:
+      str:  The state as a formated string
+    """
+    output = []
+    for state, value in self.state.items():
+      output.append('{0:s}: {1!s}'.format(state.name, value))
+    return '[{0:s}]'.format(', '.join(output))
+
   def validate(self):
     """Runs validation to verify evidence meets minimum requirements.
 
@@ -267,11 +417,14 @@ class Directory(Evidence):
 
 class CompressedDirectory(Evidence):
   """CompressedDirectory based evidence.
+
   Attributes:
     compressed_directory: The path to the compressed directory.
     uncompressed_directory: The path to the uncompressed directory.
   """
 
+  POSSIBLE_STATES = [EvidenceState.DECOMPRESSED]
+
   def __init__(
       self, compressed_directory=None, uncompressed_directory=None, *args,
       **kwargs):
@@ -281,17 +434,34 @@ class CompressedDirectory(Evidence):
     self.uncompressed_directory = uncompressed_directory
     self.copyable = True
 
-  def _preprocess(self, tmp_dir):
+  def _preprocess(self, tmp_dir, required_states):
     # Uncompress a given tar file and return the uncompressed path.
-    self.uncompressed_directory = archive.UncompressTarFile(
-        self.local_path, tmp_dir)
-    self.local_path = self.uncompressed_directory
+    if EvidenceState.DECOMPRESSED in required_states:
+      self.uncompressed_directory = archive.UncompressTarFile(
+          self.local_path, tmp_dir)
+      self.local_path = self.uncompressed_directory
+      self.state[EvidenceState.DECOMPRESSED] = True
 
   def compress(self):
-    """ Compresses a file or directory."""
+    """ Compresses a file or directory.
+
+    Creates a tar.gz from the uncompressed_directory attribute.
+    """
     # Compress a given directory and return the compressed path.
-    self.compressed_directory = archive.CompressDirectory(self.local_path)
-    self.local_path = self.compressed_directory
+    self.compressed_directory = archive.CompressDirectory(
+        self.uncompressed_directory)
+    self.source_path = self.compressed_directory
+    self.state[EvidenceState.DECOMPRESSED] = False
+
+
+class BulkExtractorOutput(CompressedDirectory):
+  """Bulk Extractor based evidence."""
+  pass
+
+
+class PhotorecOutput(CompressedDirectory):
+  """Photorec based evidence."""
+  pass
 
 
 class ChromiumProfile(Evidence):
@@ -307,37 +477,93 @@ class ChromiumProfile(Evidence):
 
   def __init__(self, browser_type=None, output_format=None, *args, **kwargs):
     """Initialization for chromium profile evidence object."""
-    super(ChromiumProfile, self).__init__(*args, **kwargs)
+    super(ChromiumProfile, self).__init__(copyable=True, *args, **kwargs)
     self.browser_type = browser_type
     self.output_format = output_format
-    self.copyable = True
 
 
 class RawDisk(Evidence):
   """Evidence object for Disk based evidence.
 
   Attributes:
-    loopdevice_path: Path to the losetup device for this disk.
-    mount_path: The mount path for this disk (if any).
+    device_path (str): Path to a relevant 'raw' data source (ie: a block
+        device or a raw disk image).
     mount_partition: The mount partition for this disk (if any).
-    size:  The size of the disk in bytes.
+    size: The size of the disk in bytes.
   """
 
-  def __init__(
-      self, mount_path=None, mount_partition=None, size=None, *args, **kwargs):
+  POSSIBLE_STATES = [EvidenceState.ATTACHED]
+
+  def __init__(self, mount_partition=1, size=None, *args, **kwargs):
     """Initialization for raw disk evidence object."""
-    self.loopdevice_path = None
-    self.mount_path = mount_path
+
+    if mount_partition < 1:
+      raise TurbiniaException(
+          'Partition numbers start at 1, but was given {0:d}'.format(
+              mount_partition))
+
+    self.device_path = None
     self.mount_partition = mount_partition
     self.size = size
     super(RawDisk, self).__init__(*args, **kwargs)
 
-  def _preprocess(self, _):
-    self.loopdevice_path = mount_local.PreprocessLosetup(self.local_path)
+  def _preprocess(self, _, required_states):
+    if EvidenceState.ATTACHED in required_states:
+      self.device_path, _ = mount_local.PreprocessLosetup(self.source_path)
+      self.state[EvidenceState.ATTACHED] = True
+      self.local_path = self.device_path
 
   def _postprocess(self):
-    mount_local.PostprocessDeleteLosetup(self.loopdevice_path)
-    self.loopdevice_path = None
+    if self.state[EvidenceState.ATTACHED]:
+      mount_local.PostprocessDeleteLosetup(self.device_path)
+      self.state[EvidenceState.ATTACHED] = False
+
+
+class RawDiskPartition(RawDisk):
+  """Evidence object for a partition within Disk based evidence.
+
+  Attributes:
+    path_spec (dfvfs.PathSpec): Partition path spec.
+    partition_offset: Offset of the partition in bytes.
+    partition_size: Size of the partition in bytes.
+  """
+
+  REQUIRED_ATTRIBUTES = ['local_path']
+  POSSIBLE_STATES = [EvidenceState.MOUNTED, EvidenceState.ATTACHED]
+
+  def __init__(
+      self, path_spec=None, partition_offset=None, partition_size=None, *args,
+      **kwargs):
+    """Initialization for raw volume evidence object."""
+
+    self.path_spec = path_spec
+    self.partition_offset = partition_offset
+    self.partition_size = partition_size
+    super(RawDiskPartition, self).__init__(*args, **kwargs)
+
+    # This Evidence needs to have a RawDisk as a parent
+    self.context_dependent = True
+
+  def _preprocess(self, _, required_states):
+    if EvidenceState.ATTACHED in required_states:
+      self.device_path, _ = mount_local.PreprocessLosetup(
+          self.source_path, partition_offset=self.partition_offset,
+          partition_size=self.partition_size)
+      if self.device_path:
+        self.state[EvidenceState.ATTACHED] = True
+        self.local_path = self.device_path
+    if EvidenceState.MOUNTED in required_states:
+      self.mount_path = mount_local.PreprocessMountPartition(self.device_path)
+      self.local_path = self.mount_path
+      self.state[EvidenceState.MOUNTED] = True
+
+  def _postprocess(self):
+    if self.state[EvidenceState.MOUNTED]:
+      mount_local.PostprocessUnmountPath(self.mount_path)
+      self.state[EvidenceState.MOUNTED] = False
+    if self.state[EvidenceState.ATTACHED]:
+      mount_local.PostprocessDeleteLosetup(self.device_path)
+      self.state[EvidenceState.ATTACHED] = False
 
 
 class EncryptedDisk(RawDisk):
@@ -349,6 +575,12 @@ class EncryptedDisk(RawDisk):
     unencrypted_path: A string to the unencrypted local path
   """
 
+  # Setting the possible states for this Evidence type explicitly to empty for
+  # now because we don't actually mount/attach these kinds of disks yet (they
+  # are currently only used by Plaso which knows how to decrypt them at
+  # runtime).
+  POSSIBLE_STATES = []
+
   def __init__(
       self, encryption_type=None, encryption_key=None, unencrypted_path=None,
       *args, **kwargs):
@@ -402,7 +634,7 @@ class APFSEncryptedDisk(EncryptedDisk):
 
 
 class GoogleCloudDisk(RawDisk):
-  """Evidence object for Google Cloud Disks.
+  """Evidence object for a Google Cloud Disk.
 
   Attributes:
     project: The cloud project name this disk is associated with.
@@ -411,6 +643,7 @@ class GoogleCloudDisk(RawDisk):
   """
 
   REQUIRED_ATTRIBUTES = ['disk_name', 'project', 'zone']
+  POSSIBLE_STATES = [EvidenceState.ATTACHED, EvidenceState.MOUNTED]
 
   def __init__(self, project=None, zone=None, disk_name=None, *args, **kwargs):
     """Initialization for Google Cloud Disk."""
@@ -420,12 +653,26 @@ class GoogleCloudDisk(RawDisk):
     super(GoogleCloudDisk, self).__init__(*args, **kwargs)
     self.cloud_only = True
 
-  def _preprocess(self, _):
-    self.local_path = google_cloud.PreprocessAttachDisk(self.disk_name)
+  def _preprocess(self, _, required_states):
+    if EvidenceState.ATTACHED in required_states:
+      self.device_path, partition_paths = google_cloud.PreprocessAttachDisk(
+          self.disk_name)
+      self.local_path = self.device_path
+      self.state[EvidenceState.ATTACHED] = True
+
+    if EvidenceState.MOUNTED in required_states:
+      self.mount_path = mount_local.PreprocessMountDisk(
+          partition_paths, self.mount_partition)
+      self.local_path = self.mount_path
+      self.state[EvidenceState.MOUNTED] = True
 
   def _postprocess(self):
-    google_cloud.PostprocessDetachDisk(self.disk_name, self.local_path)
-    self.local_path = None
+    if self.state[EvidenceState.MOUNTED]:
+      mount_local.PostprocessUnmountPath(self.mount_path)
+      self.state[EvidenceState.MOUNTED] = False
+    if self.state[EvidenceState.ATTACHED]:
+      google_cloud.PostprocessDetachDisk(self.disk_name, self.device_path)
+      self.state[EvidenceState.ATTACHED] = False
 
 
 class GoogleCloudDiskRawEmbedded(GoogleCloudDisk):
@@ -440,24 +687,49 @@ class GoogleCloudDiskRawEmbedded(GoogleCloudDisk):
     embedded_path: The path of the raw disk image inside the Persistent Disk
   """
 
-  REQUIRED_ATTRIBUTES = ['disk_name', 'project', 'zone', 'embedded_path']
+  REQUIRED_ATTRIBUTES = [
+      'disk_name', 'project', 'zone', 'embedded_partition', 'embedded_path'
+  ]
+  POSSIBLE_STATES = [
+      EvidenceState.PARENT_ATTACHED, EvidenceState.PARENT_MOUNTED
+  ]
 
-  def __init__(self, embedded_path=None, *args, **kwargs):
-    """Initialization for Google Cloud Disk."""
+  def __init__(
+      self, embedded_path=None, embedded_partition=None, *args, **kwargs):
+    """Initialization for Google Cloud Disk containing a raw disk image."""
     self.embedded_path = embedded_path
+    self.embedded_partition = embedded_partition
     super(GoogleCloudDiskRawEmbedded, self).__init__(*args, **kwargs)
 
-  def _preprocess(self, _):
-    self.local_path = google_cloud.PreprocessAttachDisk(self.disk_name)
-    self.loopdevice_path = mount_local.PreprocessLosetup(self.local_path)
-    self.mount_path = mount_local.PreprocessMountDisk(
-        self.loopdevice_path, self.mount_partition)
-    self.local_path = os.path.join(self.mount_path, self.embedded_path)
+    # This Evidence needs to have a GoogleCloudDisk as a parent
+    self.context_dependent = True
+
+  def _preprocess(self, _, required_states):
+    if EvidenceState.PARENT_ATTACHED in required_states:
+      rawdisk_path = os.path.join(
+          self.parent_evidence.mount_path, self.embedded_path)
+      if not os.path.exists(rawdisk_path):
+        raise TurbiniaException(
+            'Unable to find raw disk image {0:s} in GoogleCloudDisk'.format(
+                rawdisk_path))
+      self.device_path, partition_paths = mount_local.PreprocessLosetup(
+          rawdisk_path)
+      self.state[EvidenceState.PARENT_ATTACHED] = True
+      self.local_path = self.device_path
+
+    if EvidenceState.PARENT_MOUNTED in required_states:
+      self.mount_path = mount_local.PreprocessMountDisk(
+          partition_paths, self.mount_partition)
+      self.local_path = self.mount_path
+      self.state[EvidenceState.PARENT_MOUNTED] = True
 
   def _postprocess(self):
-    google_cloud.PostprocessDetachDisk(self.disk_name, self.local_path)
-    mount_local.PostprocessUnmountPath(self.mount_path)
-    mount_local.PostprocessDeleteLosetup(self.loopdevice_path)
+    if self.state[EvidenceState.PARENT_MOUNTED]:
+      mount_local.PostprocessUnmountPath(self.mount_path)
+      self.state[EvidenceState.PARENT_MOUNTED] = False
+    if self.state[EvidenceState.PARENT_ATTACHED]:
+      mount_local.PostprocessDeleteLosetup(self.device_path)
+      self.state[EvidenceState.PARENT_ATTACHED] = False
 
 
 class PlasoFile(Evidence):
@@ -470,18 +742,17 @@ class PlasoFile(Evidence):
   def __init__(self, plaso_version=None, *args, **kwargs):
     """Initialization for Plaso File evidence."""
     self.plaso_version = plaso_version
-    super(PlasoFile, self).__init__(*args, **kwargs)
-    self.copyable = True
+    super(PlasoFile, self).__init__(copyable=True, *args, **kwargs)
     self.save_metadata = True
 
 
-class PlasoCsvFile(PlasoFile):
+class PlasoCsvFile(Evidence):
   """Psort output file evidence.  """
 
   def __init__(self, plaso_version=None, *args, **kwargs):
     """Initialization for Plaso File evidence."""
     self.plaso_version = plaso_version
-    super(PlasoCsvFile, self).__init__(*args, **kwargs)
+    super(PlasoCsvFile, self).__init__(copyable=True, *args, **kwargs)
     self.save_metadata = False
 
 
@@ -491,8 +762,7 @@ class ReportText(Evidence):
 
   def __init__(self, text_data=None, *args, **kwargs):
     self.text_data = text_data
-    super(ReportText, self).__init__(*args, **kwargs)
-    self.copyable = True
+    super(ReportText, self).__init__(copyable=True, *args, **kwargs)
 
 
 class FinalReport(ReportText):
@@ -507,8 +777,7 @@ class TextFile(Evidence):
   """Text data."""
 
   def __init__(self, *args, **kwargs):
-    super(TextFile, self).__init__(*args, **kwargs)
-    self.copyable = True
+    super(TextFile, self).__init__(copyable=True, *args, **kwargs)
 
 
 class FilteredTextFile(TextFile):
@@ -523,9 +792,8 @@ class ExportedFileArtifact(Evidence):
 
   def __init__(self, artifact_name=None, *args, **kwargs):
     """Initializes an exported file artifact."""
-    super(ExportedFileArtifact, self).__init__(*args, **kwargs)
+    super(ExportedFileArtifact, self).__init__(copyable=True, *args, **kwargs)
     self.artifact_name = artifact_name
-    self.copyable = True
 
 
 class VolatilityReport(TextFile):
@@ -553,3 +821,40 @@ class RawMemory(Evidence):
 class BinaryExtraction(CompressedDirectory):
   """Binaries extracted from evidence."""
   pass
+
+
+class DockerContainer(Evidence):
+  """Evidence object for a DockerContainer filesystem.
+
+  Attributes:
+    container_id(str): The ID of the container to mount.
+    _container_fs_path(str): Full path to where the container filesystem will
+      be mounted.
+    _docker_root_directory(str): Full path to the docker root directory.
+  """
+
+  POSSIBLE_STATES = [EvidenceState.DOCKER_MOUNTED]
+
+  def __init__(self, container_id=None, *args, **kwargs):
+    """Initialization for Docker Container."""
+    super(DockerContainer, self).__init__(*args, **kwargs)
+    self.container_id = container_id
+    self._container_fs_path = None
+    self._docker_root_directory = None
+
+    self.context_dependent = True
+
+  def _preprocess(self, _, required_states):
+    if EvidenceState.DOCKER_MOUNTED in required_states:
+      self._docker_root_directory = GetDockerPath(
+          self.parent_evidence.mount_path)
+      # Mounting the container's filesystem
+      self._container_fs_path = docker.PreprocessMountDockerFS(
+          self._docker_root_directory, self.container_id)
+      self.mount_path = self._container_fs_path
+      self.local_path = self.mount_path
+      self.state[EvidenceState.DOCKER_MOUNTED] = True
+
+  def _postprocess(self):
+    # Unmount the container's filesystem
+    mount_local.PostprocessUnmountPath(self._container_fs_path)
